@@ -8,6 +8,7 @@
 #include "charset.h"
 
 #include "winpriv.h"  /* win_prefix_title, win_update_now */
+#include "tek.h"      /* tek_mode for log filtering */
 #include "appinfo.h"  /* APPNAME, VERSION */
 
 #include <pwd.h>
@@ -44,7 +45,7 @@ string child_dir = null;
 
 bool logging = false;
 static pid_t pid;
-static bool killed;
+static bool killed = false;
 static int win_fd;
 static int pty_fd = -1;
 static int log_fd = -1;
@@ -87,7 +88,8 @@ childerror(char * action, bool from_fork, int errno_code, int code)
 
   char s[33];
   bool colour_code = code && !errno_code;
-  sprintf(s, "\r\n\033[30;%dm\033[K", from_fork ? 41 : colour_code ? code : 43);
+  bool new_line = code || errno_code;
+  sprintf(s, "%s\033[30;%dm\033[K", new_line ? "\r\n" : "", from_fork ? 41 : colour_code ? code : 43);
   term_write(s, strlen(s));
   term_write(action, strlen(action));
   if (errno_code) {
@@ -221,6 +223,153 @@ toggle_logging()
     open_logfile(true);
 }
 
+static void
+term_log(char * s, uint len)
+{
+static char buf[999];
+static uint bufi = 0;
+static char state = 0;
+
+  void term_log_flush()
+  {
+    write(log_fd, buf, bufi);
+    bufi = 0;
+  }
+
+  void term_log_char(char c)
+  {
+    if (bufi >= sizeof(buf)) {
+      term_log_flush();
+      state = 0;
+    }
+    else if (c == '\e' && !state)
+      term_log_flush();
+
+    buf[bufi++] = c;
+
+    if (term.vt52_mode || tek_mode)
+      return;
+
+    if (!state) {
+      if (c == '') {
+        bufi--;
+        state = 0;
+      }
+      else if (c == '\e')
+        state = 'e';
+    }
+    else if (state == 'e') {
+      if (c == '[')
+        state = 'c';
+      else if (c == ']')
+        state = 'o';
+      else if (c == 'P')
+        state = 'd';
+      else if (c == 'Z') {
+        bufi = 0;
+        state = 0;
+      }
+    }
+    else if (state == 'c') { // CSI
+      if (c >= '@') { // CSI terminator
+        state = 0;
+        char * csi = &buf[2];
+        char c0 = *csi, c1 = buf[bufi - 2], c2 = buf[bufi - 1];
+        if (c0 >= '0' && c0 <= ';')
+          c0 = 0;
+        else
+          csi ++;
+        int num = 0, num2 = 0;
+        sscanf(csi, "%u;%u", &num, &num2);
+        sscanf(csi, "%u", &num);
+        if ((c2 == 'n' && (!c0 || c0 == '?'))
+            || (c2 == 'c' && !num && (!c0 || c0 == '>' || c0 == '='))
+            || (c2 == 'q' && c0 == '>' && !num)
+            || (c2 == 't' && !c0 && num >= 11 && num <= 21)
+            || (c2 == 'p' && c1 == '$' && (!c0 || c0 == '?'))
+            || (c2 == 'y' && c1 == '*' && !c0)
+            || (c2 == 'v' && c1 == '"' && !c0)
+            || (c2 == 'R' && c1 == '#' && !c0)
+            || (c2 == 'x' && !c0 && num <= 1)
+            || (c2 == 'w' && c1 == '$' && !c0)
+            || (c2 == 'm' && c0 == '?')
+            || (c2 == 'h' && c0 == '?' &&
+                (num == 9 || (num >= 1000 && num <= 1004) || num == 1007
+                 || num == 7786 || num == 7787
+                )
+               )
+            || (c2 == 'w' && c1 == '\'' && !c0)
+            || (c2 == 'z' && c1 == '\'' && !c0)
+            || (c2 == '|' && c1 == '\'' && !c0)
+            || (c2 == 'S' && c1 == '#' && !c0)
+            || (c2 == '|' && c1 == '#' && !c0)
+            || (c2 == 'S' && c0 == '?' && (num2 == 1 || num2 == 4))
+           )
+          bufi = 0;
+#ifdef debug_log_filter
+        if (!bufi)
+          printf("log filter CSI %c %d %d %c %c\n", c0 ?: '-', num, num2, c1, c2);
+#endif
+      }
+    }
+    else if (state == 'o') { // OSC
+      if (c == '' || (c == '\\' && buf[bufi - 1] == '\e')) { // ST or BEL
+        state = 0;
+        int num = -1, numq1 = -1, numq2 = -1;
+        if (2 == sscanf(&buf[2], "%u;%u;%n", &num, &numq2, &len)) {
+          if (buf[2 + len] != '?')
+            numq2 = -1;
+        }
+        else {
+          sscanf(&buf[2], "%u;%n", &num, &len);
+          if (buf[2 + len] == '?')
+            numq1 = num;
+        }
+#ifdef debug_log_filter
+        int bufend = bufi;
+#endif
+        if (numq2 == 4 || numq2 == 7704 || numq2 == 5)
+          bufi = 0;
+        else if ((numq1 >= 10 && numq1 <= 19) || numq1 == 50 || numq1 == 52
+                 || numq1 == 701 || numq1 == 7770 || numq1 == 7771 || numq1 == 7777
+                )
+          bufi = 0;
+        else if (num == 60 || num == 61 || num == 62)
+          bufi = 0;
+#ifdef debug_log_filter
+        if (!bufi) {
+          buf[bufend] = 0;
+          printf("log filter OSC %d %d %d (\\e%s)\n", num, numq1, numq2, &buf[1]);
+        }
+#endif
+      }
+    }
+    else if (state == 'd') { // DCS
+      if (c == '\\' && buf[bufi - 1] == '\e') { // ST
+        state = 0;
+        if (!strncmp(&buf[2], "$q", 2) || !strncasecmp(&buf[2], "+q", 2))
+          bufi = 0;
+      }
+    }
+  }
+
+  if (log_fd >= 0 && logging) {
+    if (!cfg.log_filter) {
+      write(log_fd, s, len);
+      return;
+    }
+    for (uint i = 0; i < len; i++)
+      term_log_char(s[i]);
+  }
+}
+
+void
+child_close_log(void)
+{
+  term_log(&(char){'\e'}, 1);  // trigger term_log_flush();
+}
+
+
 void
 child_update_charset(void)
 {
@@ -624,6 +773,12 @@ child_proc(void)
         timeout_p = &timeout;
     }
 
+#ifdef exit_WSL_after_closed_here
+    if (support_wsl && killed)
+      // force-close, as wsl.exe does not response to kill() in Windows 10
+      exit_mintty();
+#endif
+
     if (select(win_fd + 1, &fds, 0, 0, timeout_p) > 0) {
       if (pty_fd >= 0 && FD_ISSET(pty_fd, &fds)) {
         // Pty devices on old Cygwin versions (pre 1005) deliver only 4 bytes
@@ -633,8 +788,8 @@ child_proc(void)
         static char buf[4096];
         uint len = 0;
 #if CYGWIN_VERSION_API_MINOR >= 74
-        if (cfg.baud > 0) {
-          uint cps = cfg.baud / 10; // 1 start bit, 8 data bits, 1 stop bit
+        if (term.baud > 0) {
+          uint cps = term.baud / 10; // 1 start bit, 8 data bits, 1 stop bit
           uint nspc = 2000000000 / cps;
 
           static ulong prevtime = 0;
@@ -647,7 +802,7 @@ child_proc(void)
           }
           clock_gettime(CLOCK_MONOTONIC, &tim);
           ulong now = tim.tv_sec * (long)1000000000 + tim.tv_nsec;
-          //printf("baud %d ns/char %d prev %ld now %ld delta\n", cfg.baud, nspc, prevtime, now);
+          //printf("baud %d ns/char %d prev %ld now %ld delta\n", term.baud, nspc, prevtime, now);
           if (now < prevtime + nspc) {
             ulong delay = prevtime ? prevtime + nspc - now : 0;
             if (delay < exceeded)
@@ -672,15 +827,34 @@ child_proc(void)
         }
         else
 #endif
-        do {
+#if defined(collect_pty_buffer) || CYGWIN_VERSION_DLL_MAJOR < 1005
+        do
+#endif
+        {
           int ret = read(pty_fd, buf + len, sizeof buf - len);
+          //printf("%d+%d ", len, ret);
           trace_line("read", ret, buf + len, ret);
           //if (kb_trace) printf("[%lu] read %d\n", mtime(), ret);
+
           if (ret > 0)
             len += ret;
           else
             break;
-        } while (len < sizeof buf);
+
+          // The read loop buffer filling was once introduced to speed up
+          // but its implied usage of pty interferes with the WSL gateway
+          // wsl.exe and its conpty layer in some inexplicable way (#1332).
+          // (This did not happen before cygwin 3.3.5.)
+          // It is particularly completely obscure how the corrupted 
+          // display described there is triggered by entering ^O.
+          // So we disable the loop. It is apparently not needed anymore.
+          // (It could be disabled only while wsl.exe is running which
+          // could be detected by catching the DECSET 9001 sequence.)
+        }
+#if defined(collect_pty_buffer) || CYGWIN_VERSION_DLL_MAJOR < 1005
+          while (len < sizeof buf);
+#endif
+        //printf("read %d\n", len);
 
         if (len > 0) {
           term_write(buf, len);
@@ -693,8 +867,7 @@ child_proc(void)
               // undocumented safeguard in case something goes wrong here
               win_update_now();
           }
-          if (log_fd >= 0 && logging)
-            write(log_fd, buf, len);
+          term_log(buf, len);
         }
         else {
           pty_fd = -1;
@@ -714,6 +887,14 @@ child_kill(bool point_blank)
       kill(-pid, point_blank ? SIGKILL : SIGHUP) < 0 ||
       point_blank)
     exit_mintty();
+
+  if (support_wsl) {
+    //if (killed) // limit to double-close? - rather not
+
+    // force-close, as wsl.exe does not response to kill() in Windows 10
+    exit_mintty();
+  }
+
   killed = true;
 }
 
@@ -1307,7 +1488,7 @@ do_child_fork(int argc, char *argv[], int moni, bool launch, bool config_size, b
     if (pty_fd >= 0)
       close(pty_fd);
     if (log_fd >= 0)
-      close(log_fd);
+      close(log_fd);  // close file handle in forked child, no need to flush
     close(win_fd);
 
     if ((child_dir && *child_dir) || set_dir) {
