@@ -3,12 +3,18 @@
 
 typedef unsigned int ucschar;
 
+#define ZWJ 0x02
+#define ZWNJ 0x01
+
 typedef struct {
   ucschar origwc, wc;
   short index;
+  uchar wide: 1;  // width marker for term_bidi_line()
+  uchar emojilen: 7;
+  uchar joiners;
 } bidi_char;
 
-int do_bidi(bidi_char * line, int count);
+int do_bidi(bool autodir, int para_level, bool explicitRTL, bool box_mirror, bidi_char * line, int count);
 int do_shape(bidi_char * line, bidi_char * to, int count);
 
 /* bidi classes (Unicode: PropertyValueAliases.txt) */
