@@ -1,5 +1,5 @@
 // termmouse.c (part of mintty)
-// Copyright 2008-12 Andy Koppe, 2017-20 Thomas Wolff
+// Copyright 2008-2023 Andy Koppe, 2017-2025 Thomas Wolff
 // Based on code from PuTTY-0.60 by Simon Tatham and team.
 // Licensed under the terms of the GNU General Public License v3 or later.
 
@@ -37,9 +37,23 @@ static char scheme = 0;
   }
   //printf("sel_ %d: forward %d level %d\n", p.x, forward, level);
 
+  wchar prevc1 = 0;
+  wchar prevc2 = 0;
   for (;;) {
     wchar c = get_char(line, p.x);
 
+    // special colon and double-colon handling
+    if (forward && prevc1 == ':' && prevc2 != ':' && !strchr(":/", c)) {
+      // handle previous match of : if not matching :: or :/
+      return ret_p;
+    }
+    else if (forward && prevc1 == ':' && strchr(":/", c)) {
+      // handle match of :: or :/
+      ret_p = p;
+    }
+    prevc2 = prevc1;
+    prevc1 = c;
+
     // scheme detection state machine
     if (!forward) {
       // http://abc.xy
@@ -102,7 +116,17 @@ static char scheme = 0;
     // what about #$%&*\^`|~ at the end?
     else if (c == ' ' && p.x > 0 && get_char(line, p.x - 1) == '\\')
       ret_p = p;
-    else if (!(strchr("&,;?!:", c) || c == (forward ? '=' : ':'))) {
+    else if (c == (forward ? '=' : ':')) {
+    }
+    else if (strchr("&,;?!", c)) {
+    }
+    else if (forward && c == ':') {
+      // could set marker in case we match : but not :: or :/
+      // but this is better handled at the beginning of the for loop alone;
+      // note: this could be handled more easily here with some look-ahead 
+      // but that is not possible as the URL may wrap into the next line
+    }
+    else {
       //printf("%d: %c forward %d level %d BREAK\n", p.x, c, forward, level);
       break;
     }
@@ -221,6 +245,7 @@ sel_drag(pos selpoint)
 {
   //printf("sel_drag %d+%d/2 (anchor %d+%d/2)\n", selpoint.x, selpoint.r, term.sel_anchor.x, term.sel_anchor.r);
   term.selected = true;
+  term.selection_eq_clipboard = false;
   if (!term.sel_rect) {
    /*
     * For normal selection, we set (sel_start,sel_end) to
@@ -467,14 +492,14 @@ get_selpoint(const pos p)
 }
 
 static void
-send_keys(char *code, uint len, uint count)
+send_keys(uint count, string code)
 {
   if (count) {
-    uint size = len * count;
-    char buf[size];
+    uint len = strlen(code);
+    char buf[len * count];
     char *p = buf;
     while (count--) { memcpy(p, code, len); p += len; }
-    child_write(buf, size);
+    child_write(buf, sizeof buf);
   }
 }
 
@@ -585,6 +610,7 @@ term_mouse_click(mouse_button b, mod_keys mods, pos p, int count)
         p = get_selpoint(box_pos(p));
         term.mouse_state = MS_OPENING;
         term.selected = true;
+        term.selection_eq_clipboard = false;
         term.sel_rect = false;
         term.sel_start = term.sel_end = term.sel_anchor = p;
         sel_spread();
@@ -610,6 +636,7 @@ term_mouse_click(mouse_button b, mod_keys mods, pos p, int count)
       else {
         // Double or triple-click: select whole word or line
         term.selected = true;
+        term.selection_eq_clipboard = false;
         term.sel_rect = false;
         term.sel_start = term.sel_end = term.sel_anchor = p;
         sel_spread();
@@ -704,16 +731,13 @@ term_mouse_release(mouse_button b, mod_keys mods, pos p)
 
     //printf(forward ? "keys +%d\n" : "keys -%d\n", count);
     if (mode == 2003) {
-      //char erase = cfg.backspace_sends_bs ? CTRL('H') : CDEL;
       struct termios attr;
       tcgetattr(0, &attr);
-      char erase = attr.c_cc[VERASE];
-      send_keys(&erase, 1, count);
+      send_keys(count, (char[]){attr.c_cc[VERASE], 0});
     }
     else {
-      char code[3] =
-        {'\e', term.app_cursor_keys ? 'O' : '[', forward ? 'C' : 'D'};
-      send_keys(code, 3, count);
+      send_keys(count, term.app_cursor_keys ? (forward ? "\eOC" : "\eOD")
+                                            : (forward ? "\e[C" : "\e[D"));
     }
 
     moved_previously = true;
@@ -938,46 +962,87 @@ term_mouse_wheel(bool horizontal, int delta, int lines_per_notch, mod_keys mods,
       }
     }
   }
+  else if ((mods & ~(MDK_SHIFT | MDK_CTRL)) == MDK_WIN) {
+    // yes, some copy/paste here for more clarity of the conditions
+    if (strstr(cfg.suppress_wheel, "zoom"))
+      return;
+    int zoom = accu / NOTCH_DELTA;
+    if (zoom) {
+      accu -= NOTCH_DELTA * zoom;
+      win_zoom_font(zoom, mods & MDK_SHIFT);
+    }
+  }
   else if (!(mods & ~(MDK_SHIFT | MDK_CTRL | MDK_ALT))) {
-    if (mods & MDK_CTRL)
+    // Determine number of lines per wheel notch. -1 means page-wise scrolling.
+    if (mods & MDK_SHIFT)
+      lines_per_notch = -1;
+    else if (mods & MDK_CTRL)
       lines_per_notch = 1;
     else if (cfg.lines_per_notch > 0)
       lines_per_notch = min(cfg.lines_per_notch, term.rows - 1);
 
-    // Scroll, taking the lines_per_notch setting into account.
-    // Scroll by a page per notch if setting is -1 or Shift is pressed.
-    int lines_per_page = max(1, term.rows);
-    if (lines_per_notch == -1 || mods & MDK_SHIFT)
-      lines_per_notch = lines_per_page;
-    int lines = lines_per_notch * accu / NOTCH_DELTA;
-    //printf("mouse lines %d per notch %d accu %d\n", lines, lines_per_notch, accu);
-    if (lines) {
-      accu -= lines * NOTCH_DELTA / lines_per_notch;
-      if ((!term.on_alt_screen || term.show_other_screen) && !(mods & MDK_ALT)) {
+    bool pages = lines_per_notch == -1;
+    int count_per_notch = pages ? 1 : lines_per_notch;
+
+    int count = count_per_notch * accu / NOTCH_DELTA;
+
+    if (count) {
+      accu -= count * NOTCH_DELTA / count_per_notch;
+
+      bool alt = mods & MDK_ALT;
+      bool scrollback = !term.on_alt_screen || term.show_other_screen;
+
+      // If Alt is pressed while looking at the primary screen, consume the Alt
+      // modifier and send events to the application instead.
+      if (scrollback && alt)
+        scrollback = alt = false;
+
+      if (scrollback) {
         if (strstr(cfg.suppress_wheel, "scrollwin"))
           return;
-        term_scroll(0, -lines);
+        // For page-wise scrolling, scroll by one line less than window height.
+        if (pages)
+          count *= max(1, term.rows - 1);
+        term_scroll(0, -count);
       }
       else if (term.wheel_reporting || term.wheel_reporting_xterm) {
+        // Application scrolling.
         if (strstr(cfg.suppress_wheel, "scrollapp") && !term.wheel_reporting_xterm)
           return;
-        // Send scroll distance as CSI a/b events
-        bool up = lines > 0;
-        lines = abs(lines);
-        int pages = lines / lines_per_page;
-        lines -= pages * lines_per_page;
+        bool up = count > 0;
+        count = abs(count);
+
         if (term.app_wheel && !term.wheel_reporting_xterm) {
-          send_keys(up ? "\e[1;2a" : "\e[1;2b", 6, pages);
-          send_keys(up ? "\eOa" : "\eOb", 3, lines);
+          // Application wheel mode: send wheel events as CSI a/b codes rather
+          // than cursor key codes so they can be distinguished from key presses
+          // without enabling full application mouse mode.
+          // Pages are distinguished with the Shift modifier code.
+          if (pages) {
+            send_keys(count, alt ? (up ? "\e[1;4a" : "\e[1;4b")
+                                 : (up ? "\e[1;2a" : "\e[1;2b"));
+          }
+          else {
+            send_keys(count, alt ? (up ? "\e[1;3a" : "\e[1;3b")
+                                 : (up ? "\eOa" : "\eOb"));
+          }
         }
         else if (term.vt52_mode) {
-          send_keys(up ? "\eA" : "\eB", 2, lines);
+          // No PgUp/Dn keycodes in VT52 mode, so only send cursor up/down.
+          if (!pages)
+            send_keys(count, up ? "\eA" : "\eB");
         }
         else {
-          send_keys(up ? "\e[5~" : "\e[6~", 4, pages);
-          char code[3] =
-            {'\e', term.app_cursor_keys ? 'O' : '[', up ? 'A' : 'B'};
-          send_keys(code, 3, lines);
+          // Send PgUp/Dn or cursor up/down codes.
+          if (pages) {
+            send_keys(count, alt ? (up ? "\e[5;3~" : "\e[6;3~")
+                                 : (up ? "\e[5~" : "\e[6~"));
+          }
+          else {
+            send_keys(count,
+              alt ^ term.alt_wheel ? (up ? "\e[1;3A" : "\e[1;3B") :
+              term.app_cursor_keys ? (up ? "\eOA" : "\eOB")
+                                   : (up ? "\e[A" : "\e[B"));
+          }
         }
       }
     }
