@@ -1,5 +1,5 @@
 // windialog.c (part of mintty)
-// Copyright 2008-11 Andy Koppe
+// Copyright 2008-11 Andy Koppe, -2024 Thomas Wolff
 // Based on code from PuTTY-0.60 by Simon Tatham and team.
 // Licensed under the terms of the GNU General Public License v3 or later.
 
@@ -28,6 +28,29 @@ extern void setup_config_box(controlbox *);
 #include <sys/stat.h>  // chmod
 
 
+#ifdef debug_handler
+static control *
+trace_ctrl(int line, int ev, control * ctrl)
+{
+static char * debugopt = 0;
+  if (!debugopt) {
+    debugopt = getenv("MINTTY_DEBUG");
+    if (!debugopt)
+      debugopt = "";
+  }
+
+  if (strchr(debugopt, 'o')) {
+    printf("[%d ev %d] type %d (%d %d %d) label %s col %d\n", line, ev,
+           ctrl->type, !!ctrl->handler, !!ctrl->widget, !!ctrl->context,
+           ctrl->label, ctrl->column);
+  }
+  return ctrl;
+}
+// inject trace invocation into calls like ctrl->handler
+#define handler(ctrl, ev)	handler(trace_ctrl(__LINE__, ev, ctrl), ev)
+#endif
+
+
 /*
  * windlg.c - Dialogs, including the configuration dialog.
 
@@ -1001,3 +1024,10 @@ win_show_warning(char * msg)
   message_box(0, msg, null, MB_ICONWARNING, 0);
 }
 
+bool
+win_confirm_text(wchar * text, wchar * caption)
+{
+  int ret = message_box_w(0, text, caption, MB_OKCANCEL | MB_DEFBUTTON2, 0);
+  return ret == IDOK;
+}
+
